#include "Pvs.h"
#include "Timer/PerfTimer.h"
#include "common.h"
#include <iostream>


namespace CHCDemoEngine
{

void TestPvsPerformance()
{
	//  Pvs pvs(15000);
	const int prange=10000;
	Pvs pvs[prange+1];
	// test PVS performance
	const int num = 100000000;
	const int range = 50000;
	PerfTimer timer;
	timer.Entry();

	for (int i=0; i < num; i++)
	{
		int id = (int)Random(range);
		int pid = (5*num)%prange;
		pvs[pid].Insert(id, (float)i);
		if (i%1000000 == 0) {
			cout<<i/1000000<<" M"<<endl;
			cout<<"size = "<<pvs[pid].GetSize()/(1000*1000)<<"M"<<endl;
			//	  cout << "bucket_count: " << pvs.entries.bucket_count() << endl;
		}
	}
	timer.Exit();


	cout<<"Pvs insert speed = "<<num/float(1000*1000*timer.TotalTime())<<
		"M entries/s\n";
	cout<<"Memory usage "<<pvs[0].GetMemoryUsage()<<" bytes"<<endl;

	int counter = 0;
	timer.Entry();
	PvsIterator it(pvs[0]);
	for (; it.HasMoreEntries(); it.Next()) 
	{
		++ counter;
	}
	timer.Exit();

	cout<<"Pvs count = "<<counter<<endl;
	cout<<"Pvs iteration speed = "<<pvs[0].GetSize()/float(1000*1000*timer.TotalTime())<<
		"M entries/s\n";

	counter = 0;
	timer.Entry();
	PvsTimeIterator tit(pvs[0], 400);
	for (; tit.HasMoreEntries(); tit.Next()) {
		counter++;
	}
	timer.Exit();
	cout<<"Pvs time count = "<<counter<<endl;
	cout<<"Pvs iteration speed = "<<pvs[0].GetSize()/(1024*1024*timer.TotalTime())<<
		"M entries/s\n";
}

}