#include "ViewCellsTree.h"
#include "Timer/PerfTimer.h"
#include <stack>

using namespace std;


namespace CHCDemoEngine
{

ViewCell *ViewCellsTree::GetViewCell(const Vector3 &point) const
{
	ViewCellsTreeNode *node = root;

	while (!node->IsLeaf()) 
	{
		if (point[node->axis] < node->position)
			node = node->back;
		else
			node = node->front;
	}

	return node->viewCell;
}


bool ViewCellsTree::LoadFromFile(const string &filename)
{
	cout << ("Info: Loading view cells from file '" + filename + "'" );

	FILE *fr = fopen(filename.c_str(), "rb");

	if (fr == NULL) {
		Message("Error: Cannot open file for reading");
		return false;
	}

	bool result = _LoadFromFile(fr);

	Debug << "Info: done. " << endl;
	fclose(fr);

	return result;
}


bool ViewCellsTree::_LoadFromFile(FILE *fr)
{
	int buffer[256]; 
	fread(buffer, sizeof(int), 3, fr);

	if (buffer[0] != MAGIC) {
		Message( "Error: Wrong file type");
		return false;
	}

	if (buffer[1] != (int)(1000*VIEWCELLS_VERSION)) {
		Message( "Error: Wrong viewcells version" );
		return false;
	}

	// get the bounding box
	fread(&box, sizeof(AxisAlignedBox3), 1, fr);

	stack<ViewCellsTreeNode **> nodeStack;

	nodeStack.push(&root);

	while(!nodeStack.empty()) {
		ViewCellsTreeNode *&node = *nodeStack.top();
		nodeStack.pop();
		node = new ViewCellsTreeNode;
		fread(&node->axis, sizeof(int), 1, fr);
		if (!node->IsLeaf()) {
			fread(&node->position, sizeof(float), 1, fr);
			nodeStack.push(&node->front);
			nodeStack.push(&node->back);
		}
	}

	return true;
}


void ViewCellsTree::_ExportToFile(FILE *file)
{
	int buffer[256];
	int i=0;
	buffer[i++] = MAGIC;
	buffer[i++] = VIEWCELLS_VERSION*1000;

	fwrite(buffer, sizeof(int), i, file);

	fwrite(&mMaxDepth, sizeof(float), 1, file);

	fwrite(&box, sizeof(AxisAlignedBox3), 1, file);

	stack<ViewCellsTreeNode *> nodeStack;

	nodeStack.push(root);

	while (!nodeStack.empty()) {
		ViewCellsTreeNode *node = nodeStack.top();
		nodeStack.pop();

		fwrite(&node->axis, sizeof(int), 1, file);
		if (!node->IsLeaf()) {
			fwrite(&node->position, sizeof(float), 1, file);
			nodeStack.push(node->front);
			nodeStack.push(node->back);
		}
	}
}

}