#ifndef __VIEWCELLSTREE
#define __VIEWCELLSTREE

#include <vector>
#include "Pvs.h"
#include "AxisAlignedBox3.h"


	
namespace CHCDemoEngine
{

/** basic view cell structure
*/
struct ViewCell 
{
public:

	ViewCell(): mPvs() {}

protected:

	// id of this viewcell
	int mId;
	// bounding box of the viewcell
	AxisAlignedBox3 mBox;
	// the pvs associated with the viewcell
	Pvs mPvs;

};


struct ViewCellsTreeNode
{
	union 
	{
		// split position - 4B
		float position;
		// the viewcell associated with this node (NULL for non terminal nodes) - 4B
		ViewCell *viewCell; 
	};


	ViewCellsTreeNode(): mAxis(-1), mViewCell(NULL) {}

	bool IsLeaf() { return mAxis == -1; }

	bool IsVirtualLeaf() { return mViewCell != NULL; }


	///////////////////////

	// children of the node - 8B
	ViewCellsTreeNode *mBack, *mFront;

	// kd tree split params
	// split axis - 4B
	int mAxis;
};


class ViewCellsTree
{
public:
	
	ViewCellsTree(): mRoot(NULL) {}

	void SubdivideNode(ViewCellsTreeNode *node,
                       const AxisAlignedBox3 &box,
					   int depth);


	int	CastLineSegment(const Vector3 &origin,
                        const Vector3 &termination,
						ViewCell **viewcells);

	ViewCell *GetViewCell(const Vector3 &point) const;

	bool LoadFromFile(const string &filename);


protected:

	bool _LoadFromFile(FILE *file);


	////////////////////////

	float mMaxDepth;
	AxisAlignedBox3 mBox;
	ViewCellsTreeNode *mRoot;
};

}

#endif
